@@ -10,11 +10,3 @@ mod error;
 pub use error::*;
 
 pub use shell::{spawn, Context};
-
-#[allow(unused_imports)]
-#[macro_use]
-extern crate lazy_static;
-
-#[cfg(target_os = "macos")]
-#[macro_use]
-extern crate objc;
