@@ -2,7 +2,7 @@ use std::collections::HashMap;
 
 use crate::codec::Value;
 
-use super::{HotKeyHandle, MenuHandle, Point, Rect, Size};
+use super::{status_item_manager::StatusItemHandle, HotKeyHandle, MenuHandle, Point, Rect, Size};
 
 #[derive(serde::Serialize, serde::Deserialize, Debug, Clone)]
 #[serde(rename_all = "camelCase")]
@@ -73,6 +73,18 @@ impl WindowGeometryRequest {
     }
 }
 
+#[derive(serde::Deserialize, Debug, Clone)]
+#[serde(rename_all = "camelCase")]
+pub struct WindowActivateRequest {
+    pub activate_application: bool,
+}
+
+#[derive(serde::Deserialize, Debug, Clone)]
+#[serde(rename_all = "camelCase")]
+pub struct WindowDeactivateRequest {
+    pub deactivate_application: bool,
+}
+
 #[derive(serde::Deserialize, Debug, Clone)]
 #[serde(rename_all = "camelCase")]
 pub struct PopupMenuRequest {
@@ -188,6 +200,46 @@ pub struct WindowStyle {
     pub traffic_light_offset: Option<Point>,
 }
 
+#[derive(serde::Serialize, serde::Deserialize, Debug, Clone, PartialEq)]
+pub enum BoolTransition {
+    No,
+    NoToYes,
+    Yes,
+    YesToNo,
+}
+
+impl Default for BoolTransition {
+    fn default() -> Self {
+        BoolTransition::No
+    }
+}
+
+#[derive(serde::Serialize, serde::Deserialize, Debug, Clone, Default, PartialEq)]
+#[serde(rename_all = "camelCase")]
+pub struct WindowStateFlags {
+    pub maximized: BoolTransition,
+    pub minimized: BoolTransition,
+    pub full_screen: BoolTransition,
+    pub active: bool,
+}
+
+#[derive(serde::Serialize, serde::Deserialize, Debug, Clone, Default)]
+#[serde(rename_all = "camelCase")]
+pub struct WindowCollectionBehavior {
+    pub can_join_all_spaces: bool,
+    pub move_to_active_space: bool,
+    pub managed: bool,
+    pub transient: bool,
+    pub stationary: bool,
+    pub participates_in_cycle: bool,
+    pub ignores_cycle: bool,
+    pub full_screen_primary: bool,
+    pub full_screen_auxiliary: bool,
+    pub full_screen_none: bool,
+    pub allows_tiling: bool,
+    pub disallows_tiling: bool,
+}
+
 //
 // Menu
 //
@@ -195,6 +247,7 @@ pub struct WindowStyle {
 #[derive(serde::Serialize, serde::Deserialize, Debug, Clone)]
 #[serde(rename_all = "camelCase")]
 pub enum MenuItemRole {
+    About,
     Hide,
     HideOtherApplications,
     ShowAll,
@@ -329,3 +382,83 @@ pub struct HotKeyDestroyRequest {
 pub struct HotKeyPressed {
     pub handle: HotKeyHandle,
 }
+
+#[derive(serde::Serialize)]
+#[serde(rename_all = "camelCase")]
+pub struct Screen {
+    pub id: i64,
+    pub frame: Rect,
+    pub work_area: Rect,
+    pub scaling_factor: f64,
+}
+
+//
+// StatusItem
+//
+#[derive(serde::Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct StatusItemCreateRequest {}
+
+#[derive(serde::Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct StatusItemDestroyRequest {
+    pub handle: StatusItemHandle,
+}
+
+#[derive(serde::Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct StatusItemSetImageRequest {
+    pub handle: StatusItemHandle,
+    pub image: Vec<ImageData>,
+}
+
+#[derive(serde::Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct StatusItemSetHintRequest {
+    pub handle: StatusItemHandle,
+    pub hint: String,
+}
+
+#[derive(serde::Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct StatusItemSetHighlightedRequest {
+    pub handle: StatusItemHandle,
+    pub highlighted: bool,
+}
+
+#[derive(serde::Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct StatusItemShowMenuRequest {
+    pub handle: StatusItemHandle,
+    pub menu: MenuHandle,
+    pub offset: Point,
+}
+
+#[derive(serde::Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct StatusItemGetGeometryRequest {
+    pub handle: StatusItemHandle,
+}
+
+#[derive(serde::Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct StatusItemGetScreenIdRequest {
+    pub handle: StatusItemHandle,
+}
+
+#[derive(serde::Serialize)]
+#[serde(rename_all = "camelCase")]
+pub enum StatusItemActionType {
+    LeftMouseDown,
+    LeftMouseUp,
+    RightMouseDown,
+    RightMouseUp,
+}
+
+#[derive(serde::Serialize)]
+#[serde(rename_all = "camelCase")]
+pub struct StatusItemAction {
+    pub handle: StatusItemHandle,
+    pub action: StatusItemActionType,
+    pub position: Point,
+}
