@@ -1,21 +1,23 @@
 use super::{
     platform::engine::{PlatformEngine, PlatformEngineType, PlatformPlugin},
-    BinaryMessenger,
+    BinaryMessenger, EngineHandle,
 };
 use crate::Result;
 
 pub struct FlutterEngine {
     pub(super) platform_engine: PlatformEngine,
+    pub(super) parent_engine: Option<EngineHandle>,
     binary_messenger: Option<BinaryMessenger>,
 }
 
 impl FlutterEngine {
-    pub fn create(plugins: &[PlatformPlugin]) -> Self {
+    pub fn new(plugins: &[PlatformPlugin], parent_engine: Option<EngineHandle>) -> Self {
         let platform_engine = PlatformEngine::new(plugins);
 
         let messenger = BinaryMessenger::new(platform_engine.new_binary_messenger());
         FlutterEngine {
             platform_engine,
+            parent_engine,
             binary_messenger: Some(messenger),
         }
     }
