@@ -2,7 +2,6 @@ use std::fmt::Display;
 
 #[derive(Debug, Clone)]
 pub enum PlatformError {
-    NotImplemented,
     NotAvailable,
     UnknownError,
     GLibError { message: String },
@@ -14,9 +13,6 @@ pub type PlatformResult<T> = Result<T, PlatformError>;
 impl Display for PlatformError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            PlatformError::NotImplemented => {
-                write!(f, "Not Implemented")
-            }
             PlatformError::NotAvailable => {
                 write!(f, "Not Available")
             }
