@@ -1,5 +1,3 @@
-extern crate gtk;
-
 pub mod app_delegate;
 pub mod binary_messenger;
 pub mod drag_context;
@@ -14,7 +12,9 @@ pub mod keyboard_map;
 pub mod menu;
 pub mod menu_item;
 pub mod run_loop;
+pub mod screen_manager;
 pub mod size_widget;
+pub mod status_item;
 pub mod utils;
 pub mod window;
 pub mod window_menu;
