@@ -1,27 +1,29 @@
-use core::panic;
-use std::{
-    cell::{Cell, RefCell},
-    mem::ManuallyDrop,
-    rc::{Rc, Weak},
-};
-
+use super::utils::from_nsstring;
+use crate::shell::{platform::platform_impl::utils::superclass, Context, ContextRef};
+use block::{Block, RcBlock};
 use cocoa::{
     appkit::{NSApplication, NSApplicationTerminateReply},
     base::{id, nil, BOOL, NO, YES},
     foundation::{NSArray, NSUInteger},
 };
+use core::panic;
 use libc::c_void;
 use objc::{
+    class,
     declare::ClassDecl,
+    msg_send,
     rc::{autoreleasepool, StrongPtr},
     runtime::{Class, Object, Protocol, Sel},
+    sel, sel_impl,
+};
+use once_cell::sync::Lazy;
+use std::{
+    cell::{Cell, RefCell},
+    mem::ManuallyDrop,
+    rc::{Rc, Weak},
 };
 use url::Url;
 
-use crate::shell::{platform::platform_impl::utils::superclass, Context, ContextRef};
-
-use super::utils::from_nsstring;
-
 pub struct AppTermination {}
 
 impl AppTermination {
@@ -81,6 +83,14 @@ pub trait ApplicationDelegate {
     fn application_did_change_occlusion_state(&mut self) {}
 
     fn application_open_urls(&mut self, _urls: &[Url]) {}
+
+    fn application_continue_user_activity(
+        &mut self,
+        _user_activity: id, /* NSUserActivity */
+        _restoration_handler: Box<dyn FnOnce(id /* NSArray<id<NSUserActivityRestoring>> */)>,
+    ) -> bool {
+        false
+    }
 }
 
 struct DelegateState {
@@ -116,7 +126,7 @@ impl ApplicationDelegateManager {
             execute_after: RefCell::new(None),
         });
         let object = autoreleasepool(|| unsafe {
-            let object: id = msg_send![APPLICATION_DELEGATE_CLASS.0, new];
+            let object: id = msg_send![*APPLICATION_DELEGATE_CLASS, new];
             let weak = Rc::downgrade(&state);
             let state_ptr = weak.into_raw() as *mut c_void;
             (*object).set_ivar("imState", state_ptr);
@@ -150,107 +160,104 @@ impl ApplicationDelegateManager {
     }
 }
 
-struct ApplicationDelegateClass(*const Class);
-// Send is required when other dependencies apply the lazy_static feature 'spin_no_std'
-unsafe impl Send for ApplicationDelegateClass {}
-unsafe impl Sync for ApplicationDelegateClass {}
-
-lazy_static! {
-    static ref APPLICATION_DELEGATE_CLASS: ApplicationDelegateClass = unsafe {
-        let superclass = class!(NSObject);
-        let mut decl = ClassDecl::new("IMApplicationDeleagte", superclass).unwrap();
-        decl.add_ivar::<*mut c_void>("imState");
-        if let Some(protocol) = Protocol::get("NSApplicationDelegate") {
-            decl.add_protocol(protocol);
-        }
-        decl.add_method(sel!(dealloc), dealloc as extern "C" fn(&Object, Sel));
-        decl.add_method(
-            sel!(applicationWillFinishLaunching:),
-            will_finish_launching as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationDidFinishLaunching:),
-            did_finish_launching as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationWillBecomeActive:),
-            will_become_active as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationDidBecomeActive:),
-            did_become_active as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationWillResignActive:),
-            will_resign_active as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationDidResignActive:),
-            did_resign_active as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationShouldTerminate:),
-            should_terminate as extern "C" fn(&Object, Sel, id) -> NSUInteger,
-        );
-        decl.add_method(
-            sel!(applicationShouldTerminateAfterLastWindowClosed:),
-            should_terminate_after_last_window_closed as extern "C" fn(&Object, Sel, id) -> BOOL,
-        );
-        decl.add_method(
-            sel!(applicationWillTerminate:),
-            will_terminate as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationWillHide:),
-            will_hide as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationDidHide:),
-            did_hide as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationWillUnhide:),
-            will_unhide as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationDidUnhide:),
-            did_unhide as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationWillUpdate:),
-            will_update as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationDidUpdate:),
-            did_update as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationShouldHandleReopen:hasVisibleWindows:),
-            should_handle_reopen as extern "C" fn(&Object, Sel, id, BOOL) -> BOOL,
-        );
-        decl.add_method(
-            sel!(application:willPresentError:),
-            will_present_error as extern "C" fn(&Object, Sel, id, id) -> id,
-        );
-        decl.add_method(
-            sel!(applicationDidChangeScreenParameters:),
-            did_change_screen_parameters as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(applicationDidChangeOcclusionState:),
-            did_change_occlusion_state as extern "C" fn(&Object, Sel, id),
-        );
-        decl.add_method(
-            sel!(application:openFiles:),
-            open_files as extern "C" fn(&Object, Sel, id, id),
-        );
-        decl.add_method(
-            sel!(application:openURLs:),
-            open_urls as extern "C" fn(&Object, Sel, id, id),
-        );
-        ApplicationDelegateClass(decl.register())
-    };
-}
+static APPLICATION_DELEGATE_CLASS: Lazy<&'static Class> = Lazy::new(|| unsafe {
+    let superclass = class!(NSObject);
+    let mut decl = ClassDecl::new("IMApplicationDeleagte", superclass).unwrap();
+    decl.add_ivar::<*mut c_void>("imState");
+    if let Some(protocol) = Protocol::get("NSApplicationDelegate") {
+        decl.add_protocol(protocol);
+    }
+    decl.add_method(sel!(dealloc), dealloc as extern "C" fn(&Object, Sel));
+    decl.add_method(
+        sel!(applicationWillFinishLaunching:),
+        will_finish_launching as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationDidFinishLaunching:),
+        did_finish_launching as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationWillBecomeActive:),
+        will_become_active as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationDidBecomeActive:),
+        did_become_active as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationWillResignActive:),
+        will_resign_active as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationDidResignActive:),
+        did_resign_active as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationShouldTerminate:),
+        should_terminate as extern "C" fn(&Object, Sel, id) -> NSUInteger,
+    );
+    decl.add_method(
+        sel!(applicationShouldTerminateAfterLastWindowClosed:),
+        should_terminate_after_last_window_closed as extern "C" fn(&Object, Sel, id) -> BOOL,
+    );
+    decl.add_method(
+        sel!(applicationWillTerminate:),
+        will_terminate as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationWillHide:),
+        will_hide as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationDidHide:),
+        did_hide as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationWillUnhide:),
+        will_unhide as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationDidUnhide:),
+        did_unhide as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationWillUpdate:),
+        will_update as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationDidUpdate:),
+        did_update as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationShouldHandleReopen:hasVisibleWindows:),
+        should_handle_reopen as extern "C" fn(&Object, Sel, id, BOOL) -> BOOL,
+    );
+    decl.add_method(
+        sel!(application:willPresentError:),
+        will_present_error as extern "C" fn(&Object, Sel, id, id) -> id,
+    );
+    decl.add_method(
+        sel!(applicationDidChangeScreenParameters:),
+        did_change_screen_parameters as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(applicationDidChangeOcclusionState:),
+        did_change_occlusion_state as extern "C" fn(&Object, Sel, id),
+    );
+    decl.add_method(
+        sel!(application:openFiles:),
+        open_files as extern "C" fn(&Object, Sel, id, id),
+    );
+    decl.add_method(
+        sel!(application:openURLs:),
+        open_urls as extern "C" fn(&Object, Sel, id, id),
+    );
+    decl.add_method(
+        sel!(application:continueUserActivity:restorationHandler:),
+        continue_user_activity as extern "C" fn(&Object, Sel, id, id, id),
+    );
+    decl.register()
+});
 
 extern "C" fn dealloc(this: &Object, _sel: Sel) {
     unsafe {
@@ -451,6 +458,23 @@ extern "C" fn open_urls(this: &Object, _sel: Sel, _sender: id, urls: id) {
     })
 }
 
+extern "C" fn continue_user_activity(
+    this: &Object,
+    _sel: Sel,
+    _sender: id,
+    activity: id,
+    restoration_handler: id,
+) {
+    let restoration_handler = restoration_handler as *mut Block<(id,), ()>;
+    let restoration_handler = unsafe { RcBlock::copy(restoration_handler) };
+    with_delegate(this, |delegate| {
+        let f = Box::new(move |a: id| {
+            unsafe { restoration_handler.call((a,)) };
+        });
+        delegate.application_continue_user_activity(activity, f);
+    });
+}
+
 //
 
 fn with_state<F>(this: &Object, callback: F)
