@@ -1,18 +1,16 @@
 use std::{collections::HashMap, ffi::CStr};
 
+use super::utils::{from_nsstring, to_nsdata, to_nsstring};
+use crate::{
+    codec::{MessageCodec, StandardMethodCodec, Value},
+    shell::{api_constants::drag_data, ContextOptions},
+};
 use cocoa::{
     base::{id, nil},
     foundation::NSArray,
 };
 use log::warn;
-use objc::rc::StrongPtr;
-
-use crate::{
-    codec::{MessageCodec, StandardMethodCodec, Value},
-    shell::{api_constants::drag_data, ContextOptions},
-};
-
-use super::utils::{from_nsstring, to_nsdata, to_nsstring};
+use objc::{class, msg_send, rc::StrongPtr, sel, sel_impl};
 
 pub trait DragDataAdapter {
     // Retrieve data from given pasteboard
