@@ -11,6 +11,8 @@ pub mod init;
 pub mod keyboard_map;
 pub mod menu;
 pub mod run_loop;
+pub mod screen_manager;
+pub mod status_item;
 mod utils;
 pub mod window;
 
