@@ -1,25 +1,24 @@
-use std::{ffi::CString, mem::ManuallyDrop, os::raw::c_char, slice, sync::Arc};
-
+use crate::shell::{api_model::ImageData, Point, Rect, Size};
 use cocoa::{
-    appkit::{NSImage, NSView},
+    appkit::{NSImage, NSScreen, NSView},
     base::{id, nil},
     foundation::{NSArray, NSPoint, NSRect, NSSize, NSString},
 };
-
 use core_graphics::{
     base::{kCGBitmapByteOrderDefault, kCGImageAlphaLast, kCGRenderingIntentDefault},
     color_space::CGColorSpace,
     data_provider::CGDataProvider,
     image::CGImage,
 };
-
 use objc::{
+    class,
     declare::ClassDecl,
-    rc::StrongPtr,
+    msg_send,
+    rc::{autoreleasepool, StrongPtr},
     runtime::{objc_getClass, Class, Object},
+    sel, sel_impl,
 };
-
-use crate::shell::{api_model::ImageData, Point, Rect, Size};
+use std::{ffi::CString, mem::ManuallyDrop, os::raw::c_char, slice, sync::Arc};
 
 impl<'a> From<&'a Size> for NSSize {
     fn from(size: &'a Size) -> Self {
@@ -86,16 +85,14 @@ impl From<NSRect> for Rect {
     }
 }
 
-pub fn from_nsstring(ns_string: id) -> String {
-    unsafe {
-        let bytes: *const c_char = msg_send![ns_string, UTF8String];
-        let bytes = bytes as *const u8;
+pub unsafe fn from_nsstring(ns_string: id) -> String {
+    let bytes: *const c_char = msg_send![ns_string, UTF8String];
+    let bytes = bytes as *const u8;
 
-        let len = ns_string.len();
+    let len = ns_string.len();
 
-        let bytes = slice::from_raw_parts(bytes, len);
-        std::str::from_utf8(bytes).unwrap().into()
-    }
+    let bytes = slice::from_raw_parts(bytes, len);
+    std::str::from_utf8(bytes).unwrap().into()
 }
 
 pub fn to_nsstring(string: &str) -> StrongPtr {
@@ -114,6 +111,19 @@ pub fn to_nsstring(string: &str) -> StrongPtr {
 //     }
 // }
 
+pub fn global_screen_frame() -> Rect {
+    let mut res = Rect::default();
+    autoreleasepool(|| unsafe {
+        let screens = NSScreen::screens(nil);
+        for i in 0..NSArray::count(screens) {
+            let screen = NSArray::objectAtIndex(screens, i);
+            let screen_frame: Rect = NSScreen::frame(screen).into();
+            res = Rect::union(&res, &screen_frame);
+        }
+    });
+    res
+}
+
 pub fn to_nsdata(data: &[u8]) -> StrongPtr {
     unsafe {
         StrongPtr::retain(msg_send![class!(NSData), dataWithBytes:data.as_ptr() length:data.len()])
@@ -172,22 +182,21 @@ pub(super) fn class_decl_from_name(name: &str) -> ManuallyDrop<ClassDecl> {
     ManuallyDrop::new(unsafe { std::mem::transmute(res) })
 }
 
-pub(super) fn class_from_string(name: &str) -> *mut Class {
+pub(super) fn class_from_string(name: &str) -> *const Class {
     let name = CString::new(name).unwrap();
-    unsafe { objc_getClass(name.as_ptr() as *const _) as *mut _ }
+    unsafe { objc_getClass(name.as_ptr() as *const _) }
 }
 
-pub(super) fn flip_position(view: id, position: &mut NSPoint) {
-    let flipped: bool = unsafe { msg_send![view, isFlipped] };
+pub(super) unsafe fn flip_position(view: id, position: &mut NSPoint) {
+    let flipped: bool = msg_send![view, isFlipped];
     if !flipped {
-        position.y = unsafe { NSView::bounds(view) }.size.height - position.y;
+        position.y = NSView::bounds(view).size.height - position.y;
     }
 }
 
-pub(super) fn flip_rect(view: id, rect: &mut NSRect) {
-    let flipped: bool = unsafe { msg_send![view, isFlipped] };
+pub(super) unsafe fn flip_rect(view: id, rect: &mut NSRect) {
+    let flipped: bool = msg_send![view, isFlipped];
     if !flipped {
-        rect.origin.y =
-            unsafe { NSView::bounds(view) }.size.height - rect.size.height - rect.origin.y;
+        rect.origin.y = NSView::bounds(view).size.height - rect.size.height - rect.origin.y;
     }
 }
