@@ -4,6 +4,32 @@ use std::{
     rc::{Rc, Weak},
 };
 
+use windows::Win32::{
+    Foundation::{HWND, LPARAM, LRESULT, POINT, RECT, WPARAM},
+    Graphics::{
+        Dwm::DwmExtendFrameIntoClientArea,
+        Gdi::{ClientToScreen, ScreenToClient},
+    },
+    UI::{
+        Controls::MARGINS,
+        Input::KeyboardAndMouse::ReleaseCapture,
+        WindowsAndMessaging::{
+            DestroyWindow, EnableMenuItem, GetSystemMenu, GetWindowLongW, GetWindowPlacement,
+            GetWindowRect, IsWindowVisible, SendMessageW, SetForegroundWindow, SetWindowLongW,
+            SetWindowPlacement, SetWindowPos, SetWindowTextW, ShowWindow, GWL_EXSTYLE, GWL_STYLE,
+            HTBOTTOM, HTBOTTOMLEFT, HTBOTTOMRIGHT, HTCAPTION, HTCLIENT, HTLEFT, HTRIGHT, HTTOP,
+            HTTOPLEFT, HTTOPRIGHT, HTTRANSPARENT, HWND_BOTTOM, MF_BYCOMMAND, MF_DISABLED,
+            MF_ENABLED, MF_GRAYED, SC_CLOSE, SHOW_WINDOW_CMD, SWP_FRAMECHANGED, SWP_NOACTIVATE,
+            SWP_NOMOVE, SWP_NOSIZE, SWP_NOZORDER, SW_HIDE, SW_SHOW, WINDOWPLACEMENT, WINDOWPOS,
+            WINDOW_EX_STYLE, WINDOW_STYLE, WM_CLOSE, WM_DESTROY, WM_DISPLAYCHANGE,
+            WM_DWMCOMPOSITIONCHANGED, WM_NCCALCSIZE, WM_NCHITTEST, WM_NCLBUTTONDOWN,
+            WM_WINDOWPOSCHANGING, WS_BORDER, WS_CAPTION, WS_DLGFRAME, WS_EX_LAYOUTRTL,
+            WS_EX_NOREDIRECTIONBITMAP, WS_MAXIMIZEBOX, WS_MINIMIZEBOX, WS_OVERLAPPEDWINDOW,
+            WS_POPUP, WS_SYSMENU, WS_THICKFRAME,
+        },
+    },
+};
+
 use crate::{
     shell::{
         api_model::{
@@ -16,11 +42,10 @@ use crate::{
 };
 
 use super::{
-    all_bindings::*,
     display::Displays,
     error::PlatformResult,
     flutter_sys::{FlutterDesktopGetDpiForHWND, FlutterDesktopGetDpiForMonitor},
-    util::{as_u8_slice, clamp, BoolResultExt, GET_X_LPARAM, GET_Y_LPARAM},
+    util::{as_u8_slice, BoolResultExt, GET_X_LPARAM, GET_Y_LPARAM},
 };
 
 pub struct WindowBaseState {
@@ -32,6 +57,7 @@ pub struct WindowBaseState {
     delegate: Weak<dyn WindowDelegate>,
     style: RefCell<WindowStyle>,
     pending_show_cmd: Cell<SHOW_WINDOW_CMD>,
+    last_window_pos: RefCell<Option<WINDOWPOS>>,
 }
 
 const LARGE_SIZE: f64 = 64.0 * 1024.0;
@@ -47,6 +73,7 @@ impl WindowBaseState {
             max_content_size: RefCell::new(Size::wh(LARGE_SIZE, LARGE_SIZE)),
             style: Default::default(),
             pending_show_cmd: Cell::new(SW_SHOW),
+            last_window_pos: RefCell::new(None),
         }
     }
 
@@ -58,6 +85,21 @@ impl WindowBaseState {
         unsafe { Ok(SetForegroundWindow(self.hwnd).into()) }
     }
 
+    pub fn deactivate(&self) -> PlatformResult<bool> {
+        unsafe {
+            Ok(SetWindowPos(
+                self.hwnd,
+                HWND_BOTTOM,
+                0,
+                0,
+                0,
+                0,
+                SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE,
+            )
+            .into())
+        }
+    }
+
     pub fn show<F>(&self, callback: F) -> PlatformResult<()>
     where
         F: FnOnce() + 'static,
@@ -291,8 +333,8 @@ impl WindowBaseState {
             std::cmp::min(max_content.height, max_frame.height),
         );
 
-        position.cx = clamp(position.cx, min_size.width, max_size.width);
-        position.cy = clamp(position.cy, min_size.height, max_size.height);
+        position.cx = position.cx.clamp(min_size.width, max_size.width);
+        position.cy = position.cy.clamp(min_size.height, max_size.height);
 
         Ok(())
     }
@@ -567,12 +609,28 @@ impl WindowBaseState {
             }
             WM_DISPLAYCHANGE => {
                 Displays::displays_changed();
-                self.delegate().displays_changed();
                 None
             }
             WM_WINDOWPOSCHANGING => {
                 let position = unsafe { &mut *(l_param.0 as *mut WINDOWPOS) };
+                let pos_before = *position;
                 self.adjust_window_position(position).ok_log();
+
+                if let Some(ref prev_window_pos) = *self.last_window_pos.borrow() {
+                    if pos_before.cx < position.cx {
+                        // fix window drift when resizing left border past minimum size
+                        if position.x != prev_window_pos.x {
+                            position.x = prev_window_pos.x + prev_window_pos.cx - position.cx;
+                        }
+                    }
+                    if pos_before.cy < position.cy {
+                        // fix window drift when resizing top border past minimum size
+                        if position.y != prev_window_pos.y {
+                            position.y = prev_window_pos.y + prev_window_pos.cy - position.cy;
+                        }
+                    }
+                }
+                self.last_window_pos.borrow_mut().replace(*position);
                 None
             }
             WM_DWMCOMPOSITIONCHANGED => {
@@ -589,7 +647,7 @@ impl WindowBaseState {
             WM_NCHITTEST => {
                 if self.remove_border() {
                     let res = self.do_hit_test(GET_X_LPARAM(l_param), GET_Y_LPARAM(l_param));
-                    Some(LRESULT(res as i32))
+                    Some(LRESULT(res as isize))
                 } else {
                     None
                 }
@@ -610,9 +668,9 @@ impl WindowBaseState {
                 if self.remove_border() {
                     let res = self.do_hit_test(GET_X_LPARAM(l_param), GET_Y_LPARAM(l_param));
                     if res != HTCLIENT {
-                        Some(LRESULT(HTTRANSPARENT))
+                        Some(LRESULT(HTTRANSPARENT as isize))
                     } else {
-                        Some(LRESULT(res as i32))
+                        Some(LRESULT(res as isize))
                     }
                 } else {
                     None
@@ -626,5 +684,4 @@ impl WindowBaseState {
 pub trait WindowDelegate {
     fn should_close(&self);
     fn will_close(&self);
-    fn displays_changed(&self);
 }
